@@ -1,6 +1,6 @@
 /* hexter DSSI software synthesizer GUI
  *
- * Copyright (C) 2004, 2009 Sean Bolton and others.
+ * Copyright (C) 2004, 2009, 2012 Sean Bolton and others.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,11 +45,12 @@
 #include "gui_data.h"
 #include "gui_interface.h"
 #include "gui_midi.h"
+#include "gui_patch_edit.h"
 
 static int internal_gui_update_only = 0;
 
 static unsigned char test_note_noteon_key = 60;
-static unsigned char test_note_noteoff_key;
+static int           test_note_noteoff_key = -1;
 static unsigned char test_note_velocity = 84;
 
 static int export_file_type;
@@ -111,6 +112,16 @@ on_menu_quit_activate                  (GtkMenuItem     *menuitem,
 }
 
 
+void
+on_menu_edit_activate(GtkMenuItem *menuitem, gpointer user_data)
+{
+    if (!GTK_WIDGET_MAPPED(editor_window))
+        gtk_widget_show(editor_window);
+    else
+        gdk_window_raise(editor_window->window);
+}
+
+
 void
 on_menu_about_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -170,12 +181,12 @@ on_import_file_cancel( GtkWidget *widget, gpointer data )
 void
 on_position_change(GtkWidget *widget, gpointer data)
 {
-    char name[11];
+    char name[21];
 
     int position = lrintf(GTK_ADJUSTMENT(widget)->value);
     GtkWidget *label = (GtkWidget *)data;
 
-    dx7_voice_copy_name(name, &patches[position]);
+    patch_edit_copy_name_to_utf8(name, patches[position].data, TRUE);
     gtk_label_set_text (GTK_LABEL (label), name);
 }
 
@@ -193,6 +204,11 @@ on_import_file_position_ok( GtkWidget *widget, gpointer data )
 
         /* successfully loaded at least one patch */
         rebuild_patches_clist();
+        if (!edit_buffer_active) {
+            dx7_patch_unpack(patches, edit_buffer.program, edit_buffer.voice);
+            /* set all the patch edit widgets to match */
+            patch_edit_update_editors();
+        }
         display_notice("Load Patch File succeeded:", message);
         gui_data_send_dirty_patch_sections();
 
@@ -256,7 +272,7 @@ on_export_file_position_change(GtkWidget *widget, gpointer data)
     int type = GTK_TOGGLE_BUTTON (export_file_type_sysex)->active ? 0 : 1;
     int start = lrintf(GTK_ADJUSTMENT(export_file_start_spin_adj)->value);
     int end   = lrintf(GTK_ADJUSTMENT(export_file_end_spin_adj)->value);
-    char name[11];
+    char name[21];
 
     if (which == 0) {  /* start */
         if (type == 0) {  /* sys-ex */
@@ -270,7 +286,7 @@ on_export_file_position_change(GtkWidget *widget, gpointer data)
                 gtk_signal_emit_by_name (GTK_OBJECT (export_file_end_spin_adj), "value_changed");
             }
         }
-        dx7_voice_copy_name(name, &patches[start]);
+        patch_edit_copy_name_to_utf8(name, patches[start].data, TRUE);
         gtk_label_set_text (GTK_LABEL (export_file_start_name), name);
     } else { /* end */
         if (type == 1) {  /* raw */
@@ -279,7 +295,7 @@ on_export_file_position_change(GtkWidget *widget, gpointer data)
                 gtk_signal_emit_by_name (GTK_OBJECT (export_file_start_spin_adj), "value_changed");
             }
         }
-        dx7_voice_copy_name(name, &patches[end]);
+        patch_edit_copy_name_to_utf8(name, patches[end].data, TRUE);
         gtk_label_set_text (GTK_LABEL (export_file_end_name), name);
     }
 }
@@ -360,21 +376,17 @@ on_patches_selection(GtkWidget      *clist,
 
     GUIDB_MESSAGE(DB_GUI, " on_patches_selection: patch %d selected\n", current_program);
 
-    /* set all the patch edit widgets to match */
-    /* update_voice_widgets_from_patch(&patches[current_program]); */
-
-    lo_send(osc_host_address, osc_program_path, "ii", 0, current_program);
-
-    if (edit_buffer_active && current_program != edit_buffer.program) {
-
-        gui_data_clear_edit_buffer();
-
-        gtk_label_set_text (GTK_LABEL (sysex_status_label), " No active edits.");
-        gtk_widget_set_sensitive (sysex_discard_button, FALSE);
-        gtk_widget_set_sensitive (sysex_save_button, FALSE);
-
-        edit_buffer_active = 0;
+    if (current_program != edit_buffer.program) {
+        if (edit_buffer_active) {
+            gui_data_send_edit_buffer_off();
+            edit_buffer_active = FALSE;
+        }
+        edit_buffer.program = current_program;
+        dx7_patch_unpack(patches, edit_buffer.program, edit_buffer.voice);
+        /* set all the patch edit widgets to match */
+        patch_edit_update_editors(); /* also updates status */
     }
+    lo_send(osc_host_address, osc_program_path, "ii", 0, current_program);
 }
 
 void
@@ -463,29 +475,39 @@ on_compat059_toggled(GtkWidget *widget, gpointer data)
     send_performance();
 }
 
-#ifdef MIDI_ALSA
+#ifdef HEXTER_DEBUG_CONTROL
 void
-on_sysex_receipt(unsigned int length, unsigned char* data)
+on_test_changed(GtkObject *adj, gpointer data)
 {
-    int had_active_edits = edit_buffer_active;
+    int value = (int)GTK_ADJUSTMENT(adj)->value;
+    unsigned char midi[4];
 
-    GUIDB_MESSAGE(DB_IO, " on_sysex_receipt: message length %d, mfg id %02x\n", length, data[1]);
-
-    if (gui_data_sysex_parse(length, data)) {
-
-        gui_data_send_edit_buffer();
-
-        if (!had_active_edits) {
-            char buf[32];
-
-            snprintf(buf, 32, " Edits overlaying patch %d!", edit_buffer.program);
-            gtk_label_set_text (GTK_LABEL (sysex_status_label), buf);
-            gtk_widget_set_sensitive (sysex_discard_button, TRUE);
-            gtk_widget_set_sensitive (sysex_save_button, TRUE);
-        }
-    }
+#if 1
+    midi[0] = 0;
+    midi[1] = 0xB0; /* control change */
+    midi[2] = 0x63; /* NRPN MSB */
+    midi[3] = 0;
+    lo_send(osc_host_address, osc_midi_path, "m", midi);
+    midi[2] = 0x62; /* NRPN LSB */
+    midi[3] = 40;
+    lo_send(osc_host_address, osc_midi_path, "m", midi);
+    midi[2] = 0x06; /* Data Entry MSB */
+    midi[3] = (value / 128) & 127;
+    lo_send(osc_host_address, osc_midi_path, "m", midi);
+    midi[2] = 0x26; /* Data Entry LSB */
+    midi[3] = value & 127;
+    lo_send(osc_host_address, osc_midi_path, "m", midi);
+#else
+    midi[0] = 0;
+    midi[1] = 0xB0; /* control change */
+    midi[2] = 0x50; /* GP5 */
+    midi[3] = (value / 128) & 127;
+    lo_send(osc_host_address, osc_midi_path, "m", midi);
+#endif
 }
+#endif /* HEXTER_DEBUG_CONTROL */
 
+#ifdef MIDI_ALSA
 void
 on_sysex_enable_toggled( GtkWidget *widget, gpointer data )
 {
@@ -511,14 +533,9 @@ on_sysex_enable_toggled( GtkWidget *widget, gpointer data )
 
             gtk_widget_set_sensitive (sysex_channel_label, TRUE);
             gtk_widget_set_sensitive (sysex_channel_spin, TRUE);
+            snprintf(buf, 32, " Listening on port %d:%d", sysex_seq_client_id, sysex_seq_port_id);
+            gtk_label_set_text (GTK_LABEL (sysex_status_label), buf);
 
-#ifdef MIDI_ALSA
-            if (!edit_buffer_active) {
-                snprintf(buf, 32, " (listening at %d:%d)", sysex_seq_client_id, sysex_seq_port_id);
-                gtk_label_set_text (GTK_LABEL (sysex_status_label), buf);
-            }
-#endif /* MIDI_ALSA */
-            
         } else {
 
             display_notice("Error: could not start MIDI client for sys-ex reception.", message);
@@ -538,48 +555,17 @@ on_sysex_enable_toggled( GtkWidget *widget, gpointer data )
 
         gtk_widget_set_sensitive (sysex_channel_label, FALSE);
         gtk_widget_set_sensitive (sysex_channel_spin, FALSE);
+        gtk_label_set_text (GTK_LABEL (sysex_status_label), "");
     }
 }
 
 void
 on_sysex_channel_change(GtkWidget *widget, gpointer data)
 {
-    edit_receive_channel = lrintf(GTK_ADJUSTMENT(widget)->value) - 1;
+    sysex_receive_channel = lrintf(GTK_ADJUSTMENT(widget)->value) - 1;
 
     GUIDB_MESSAGE(DB_GUI, " on_sysex_channel_change: channel now %d\n",
-                  edit_receive_channel + 1);
-}
-
-void
-on_sysex_discard_button_press(GtkWidget *widget, gpointer data)
-{
-    GUIDB_MESSAGE(DB_GUI, ": on_sysex_discard_button_press called\n");
-
-    gtk_widget_hide(edit_save_position_window);
-
-    gui_data_clear_edit_buffer();
-
-    gtk_label_set_text (GTK_LABEL (sysex_status_label), " No active edits.");
-    gtk_widget_set_sensitive (sysex_discard_button, FALSE);
-    gtk_widget_set_sensitive (sysex_save_button, FALSE);
-
-    edit_buffer_active = 0;
-}
-
-void
-on_sysex_save_button_press(GtkWidget *widget, gpointer data)
-{
-    GUIDB_MESSAGE(DB_GUI, ": on_sysex_save_button_press called\n");
-    
-    /* (GTK_ADJUSTMENT(edit_save_position_spin_adj))->value = 
-     *         (patch_count == 128 ?   0.0f : (float)patch_count);
-     * (GTK_ADJUSTMENT(edit_save_position_spin_adj))->upper =
-     *         (patch_count == 128 ? 127.0f : (float)patch_count); */
-
-    /* update patch name */
-    gtk_signal_emit_by_name (GTK_OBJECT (edit_save_position_spin_adj), "value_changed");
-
-    gtk_widget_show(edit_save_position_window);
+                  sysex_receive_channel + 1);
 }
 #endif /* MIDI_ALSA */
 
@@ -592,19 +578,29 @@ on_edit_save_position_ok(GtkWidget *widget, gpointer data)
 
     GUIDB_MESSAGE(DB_GUI, " on_edit_save_position_ok: position %d\n", position);
 
-    dx7_patch_pack(edit_buffer.buffer, patches, position);
+    /* Hmm, while the edit save position dialog is shown, there's nothing
+     * to prevent program changes or patch configures from causing the edit
+     * buffer to be overwritten.... */
 
+    edit_buffer.program = position;
+    dx7_patch_pack(edit_buffer.voice, patches, position);
     gui_data_mark_dirty_patch_sections(position, position);
     gui_data_send_dirty_patch_sections();
-    gui_data_clear_edit_buffer();
 
-    gtk_label_set_text (GTK_LABEL (sysex_status_label), " No active edits.");
-    gtk_widget_set_sensitive (sysex_discard_button, FALSE);
-    gtk_widget_set_sensitive (sysex_save_button, FALSE);
+    /* We make the saved-to program the new current one, and clear the
+     * edit_buffer overlay if active, because this makes the system's
+     * state more intuitively understandable. */
+    if (position != current_program) {
+        current_program = position;
+        lo_send(osc_host_address, osc_program_path, "ii", 0, current_program);
+    }
+    if (edit_buffer_active) {
+        edit_buffer_active = FALSE;
+        gui_data_send_edit_buffer_off();
+    }
 
     rebuild_patches_clist();
-
-    edit_buffer_active = 0;
+    patch_edit_update_status();
 }
 
 void
@@ -620,7 +616,7 @@ send_performance(void)
     uint8_t perf_buffer[DX7_PERFORMANCE_SIZE];
     uint8_t p;
 
-    memcpy(perf_buffer, dx7_init_performance, DX7_PERFORMANCE_SIZE);
+    hexter_data_performance_init(perf_buffer);
 
     perf_buffer[0]  = (GTK_TOGGLE_BUTTON (compat059_button)->active) ? 1 : 0;  /* 0.5.9 compatibility */
 
@@ -659,7 +655,7 @@ on_performance_spin_change(GtkWidget *widget, gpointer data)
         return;
     }
 
-    GUIDB_MESSAGE(DB_GUI, " on_performance_spin_change: '%s' set to %d\n",
+    GUIDB_MESSAGE(DB_GUI, " on_performance_spin_change: '%s' set to %ld\n",
                   performance_spin_names[(int)data],
                   lrintf(GTK_ADJUSTMENT(widget)->value));
 
@@ -676,9 +672,9 @@ on_performance_assign_toggled(GtkWidget *widget, gpointer data)
 
     GUIDB_MESSAGE(DB_GUI, " on_performance_assign_toggled: '%s' now P%d A%d E%d\n",
                   performance_assign_names[(int)data],
-                  GTK_TOGGLE_BUTTON (performance_assign_widgets[parameter][0])->active ? 1 : 0,
-                  GTK_TOGGLE_BUTTON (performance_assign_widgets[parameter][1])->active ? 1 : 0,
-                  GTK_TOGGLE_BUTTON (performance_assign_widgets[parameter][2])->active ? 1 : 0);
+                  GTK_TOGGLE_BUTTON (performance_assign_widgets[(int)data][0])->active ? 1 : 0,
+                  GTK_TOGGLE_BUTTON (performance_assign_widgets[(int)data][1])->active ? 1 : 0,
+                  GTK_TOGGLE_BUTTON (performance_assign_widgets[(int)data][2])->active ? 1 : 0);
 
     send_performance();
 }
@@ -701,27 +697,44 @@ on_test_note_slider_change(GtkWidget *widget, gpointer data)
     }
 }
 
-void
-on_test_note_button_press(GtkWidget *widget, gpointer data)
+static void
+send_midi(unsigned char b0, unsigned char b1, unsigned char b2)
 {
     unsigned char midi[4];
 
-    if ((int)data) {  /* button pressed */
+    midi[0] = 0;
+    midi[1] = b0;
+    midi[2] = b1;
+    midi[3] = b2;
+    lo_send(osc_host_address, osc_midi_path, "m", midi);
+}
 
-        midi[0] = 0;
-        midi[1] = 0x90;
-        midi[2] = test_note_noteon_key;
-        midi[3] = test_note_velocity;
-        lo_send(osc_host_address, osc_midi_path, "m", midi);
-        test_note_noteoff_key = test_note_noteon_key;
+void
+release_test_note(void)
+{
+    if (test_note_noteoff_key >= 0) {
+        send_midi(0x80, test_note_noteoff_key, 0x40);
+        test_note_noteoff_key = -1;
+    }
+}
 
+void
+on_test_note_button_press(GtkWidget *widget, gpointer data)
+{
+    int state = (int)data;
+
+    GUIDB_MESSAGE(DB_GUI, " on_test_note_button_press: button %s\n",
+                  state ? "pressed" : "released")
+
+    if (state) {  /* button pressed */
+
+        if (test_note_noteoff_key < 0) {
+            send_midi(0x90, test_note_noteon_key, test_note_velocity);
+            test_note_noteoff_key = test_note_noteon_key;
+        }
     } else { /* button released */
 
-        midi[0] = 0;
-        midi[1] = 0x80;
-        midi[2] = test_note_noteoff_key;
-        midi[3] = 0x40;
-        lo_send(osc_host_address, osc_midi_path, "m", midi);
+        release_test_note();
 
     }
 }
@@ -786,21 +799,22 @@ update_from_program_select(unsigned long bank, unsigned long program)
 
         current_program = program;
 
-        internal_gui_update_only = 1;
-        gtk_clist_select_row (GTK_CLIST(patches_clist), program, 0);
-        internal_gui_update_only = 0;
+        patches_clist_set_program();
 
-        /* update_voice_widgets_from_patch(&patches[program]); */
-
-        if (edit_buffer_active && program != edit_buffer.program) {
-
-            gui_data_clear_edit_buffer();
-
-            gtk_label_set_text (GTK_LABEL (sysex_status_label), " No active edits.");
-            gtk_widget_set_sensitive (sysex_discard_button, FALSE);
-            gtk_widget_set_sensitive (sysex_save_button, FALSE);
-
-            edit_buffer_active = 0;
+        /* If we receive a program change that doesn't match what's in the
+         * edit buffer, then we clear the edit buffer. Note that the converse
+         * is not done: if we receive an edit_buffer configure whose program
+         * number does not match current_program, we don't change the program.
+         */
+        if (program != edit_buffer.program) {
+            if (edit_buffer_active) {
+                gui_data_send_edit_buffer_off();
+                edit_buffer_active = FALSE;
+            }
+            edit_buffer.program = program;
+            dx7_patch_unpack(patches, edit_buffer.program, edit_buffer.voice);
+            /* set all the patch edit widgets to match */
+            patch_edit_update_editors(); /* also updates status */
         }
 
     } else {  /* out of range */
@@ -829,9 +843,12 @@ update_patches(const char *key, const char *value)
     patch_section_dirty[section] = 0;
 
     rebuild_patches_clist();
-    internal_gui_update_only = 1;
-    gtk_clist_select_row (GTK_CLIST(patches_clist), current_program, 0);
-    internal_gui_update_only = 0;
+
+    if (!edit_buffer_active && (edit_buffer.program / 32) == section) {
+        dx7_patch_unpack(patches, edit_buffer.program, edit_buffer.voice);
+        /* set all the patch edit widgets to match */
+        patch_edit_update_editors();
+    }
 }
 
 void
@@ -841,35 +858,19 @@ update_edit_buffer(const char *value)
     
     if (!strcmp(value, "off")) {
 
-        if (edit_buffer_active) {
-            if (sysex_enabled) {
-                gtk_label_set_text (GTK_LABEL (sysex_status_label), " No active edits.");
-            } else {
-                gtk_label_set_text (GTK_LABEL (sysex_status_label), " ");
-            }
-            gtk_widget_set_sensitive (sysex_discard_button, FALSE);
-            gtk_widget_set_sensitive (sysex_save_button, FALSE);
-
-            edit_buffer_active = 0;
-        }
+        edit_buffer.program = current_program;
+        dx7_patch_unpack(patches, current_program, edit_buffer.voice);
+        edit_buffer_active = FALSE;
+        patch_edit_update_editors();
 
     } else {
-        
-        char buf[32];
 
         if (!decode_7in6(value, sizeof(edit_buffer), (uint8_t *)&edit_buffer)) {
             GUIDB_MESSAGE(DB_OSC, " update_edit_buffer: corrupt data!\n");
             return;
         }
-
-        snprintf(buf, 32, " Edits overlaying patch %d!", edit_buffer.program);
-        gtk_label_set_text (GTK_LABEL (sysex_status_label), buf);
-        if (!edit_buffer_active) {
-            gtk_widget_set_sensitive (sysex_discard_button, TRUE);
-            gtk_widget_set_sensitive (sysex_save_button, TRUE);
-        }
-
-        edit_buffer_active = 1;
+        edit_buffer_active = TRUE;
+        patch_edit_update_editors();
     }
 }
 
@@ -1007,10 +1008,22 @@ update_global_polyphony(const char *value)
     }
 }
 
+void
+patches_clist_set_program(void)
+{
+    /* set active row */
+    internal_gui_update_only = 1;
+    gtk_clist_select_row (GTK_CLIST(patches_clist), current_program, 0);
+    internal_gui_update_only = 0;
+    /* scroll window to show active row */
+    double d = (current_program == 0) ? 0.0 : 0.5; /* clist buggy? no! (gtk_clist_row_is_visible doesn't work right, either) */
+    gtk_clist_moveto(GTK_CLIST(patches_clist), current_program, 0, d, 0);
+}
+
 void
 rebuild_patches_clist(void)
 {
-    char number[4], name[11];
+    char number[4], name[21];
     char *data[2] = { number, name };
     int i;
 
@@ -1020,13 +1033,15 @@ rebuild_patches_clist(void)
     gtk_clist_clear(GTK_CLIST(patches_clist));
     for (i = 0; i < 128; i++) {
         snprintf(number, 4, "%d", i);
-        dx7_voice_copy_name(name, &patches[i]);
+        patch_edit_copy_name_to_utf8(name, patches[i].data, TRUE);
         gtk_clist_append(GTK_CLIST(patches_clist), data);
     }
-#if GTK_CHECK_VERSION(2, 0, 0)
+
     /* kick GTK+ 2.4.x in the pants.... */
     gtk_signal_emit_by_name (GTK_OBJECT (patches_clist), "check-resize");
-#endif    
+
     gtk_clist_thaw(GTK_CLIST(patches_clist));
+
+    patches_clist_set_program();
 }
 
